<?php

/**
 * @file
 * Implementation of VoIP validation classes
 */

class VoipNumber {

  private $number;
  private $name;
  private $type;
  private $nid;
  private $uid;

  function __construct($number, $name, $type) {
    $this->number = $number;
    $this->name = $name;
    $this->type = $type;
  }

  /**
   * Returns an array with the VoIP Numbers associated with the given user
   *
   * @param $uid
   *   Integer representing user id value. If left empty then current user id will be used instead.
   *
   * @return
   *   Array of VoIP number objects.
   */
  public static function getNumbersFromUid($uid = NULL) {
    if (empty($uid)) {
      global $user;
      $uid = $user->uid;
    }
    //Invoke hook_get_voip_numbers()
    $numbers = module_invoke_all('get_voip_numbers', $uid, 'user');
    return $numbers;
  }

  /**
   * Returns the default VoIP Number associated with the given user
   *
   * @param $uid
   *   Integer representing user id value. If left empty then current user id will be used instead.
   *
   * @return
   *   Single VoIP number object.
   */
  public static function getDefaultNumberFromUid($uid = NULL) {
    if (empty($uid)) {
      global $user;
      $uid = $user->uid;
    }
    $voip_user = user_load($uid);
    //Retrieve default either from user profile
    if (!empty($voip_user->voipnumber_phone)) {
      $number = $voip_user->voipnumber_country . $voip_user->voipnumber_phone;
      $name = 'Profile number(' . $numbers[0]->number . ')';
      $type = 'VoIP User Profile Number';
      $vn = new VoipNumber($number, $name, $type);
      return $vn;
    }
    else {
      //otherwise, get first number from the array
      $numbers = VoipNumber::getNumbersFromUid($uid);
      if (empty($numbers)) {
        return NULL;
      }
    }
    return $numbers[0];
  }

  /**
   * Returns an array with the VoIP Numbers associated with the given node
   *
   * @param $nid
   *   Integer representing node id value.
   *
   * @return
   *   Array of VoIP number objects.
   */
  public static function getNumbersFromNid($nid) {
    //Invoke hook_get_voip_numbers()
    $numbers = module_invoke_all('get_voip_numbers', $nid, 'node');
    return $numbers;
  }

  /**
   * Returns the default VoIP Number associated with the given node
   *
   * @param $nid
   *   Integer representing node id value.
   *
   * @return
   *   Single VoIP number object.
   */
  public static function getDefaultNumberFromNid($nid) {
    $numbers = VoipNumber::getNumbersFromNid($nid);
    if (empty($numbers)) {
      return NULL;
    }
    return $numbers[0];
  }

  /**
   * Returns an array with the user ids associated with the specified number
   *
   * @param $number
   *   String containing the number
   *
   * @return
   *   Array of user ids.
   */
  public static function getUidsFromNumber($number) {
    $number = VoipNumber::NormalizeNumber($number);
    $uids = module_invoke_all('get_voip_numbers', $number, 'uids');
    return $uids;
  }


  /**
   * Returns an array with the node ids associated with the given number
   *
   * @param $number
   *   String containing the number
   *
   * @return
   *   Array of node ids.
   */
  public static function getNidsFromNumber($number) {
    $number = VoipNumber::NormalizeNumber($number);
    $nids = module_invoke_all('get_voip_numbers', $number, 'nids');
    return $nids;
  }

  function getNumber() {
    return $this->number;
  }

  function getType() {
    return $this->type;
  }

  function getName() {
    return $this->name;
  }

  function getUid() {
    return $this->uid;
  }

  function getNid() {
    return $this->nid;
  }

  /**
   * Returns string with the country code
   *
   * @param $country
   *   String containing the country name
   *
   * @return
   *   string with the country code
   */
  public static function getCodeFromCountry($country) {
    $code = db_result(db_query("SELECT code FROM {voipnumber_dialing_codes} WHERE LOWER(country) ='%s'", strtolower($country)));
    return $code;
  }

  /**
   * Returns string with the country name or an array of country names
   *
   * @param $code
   *   Country code in any of following formats(1, +1, 001)
   *
   * @return
   *   string or array with country names
   */
  public static function getCountryFromCode($code) {
    $code = VoipNumber::NormalizeNumber($code);
    $country = array();
    $query = db_query("SELECT country FROM {voipnumber_dialing_codes} WHERE code='%s'", $code);
    while($row = db_fetch_array($query)) {
      $country[] = $row['country'];
    }
    if(empty($country)) {
      return NULL;
    }
    else if(count($country) === 1) {
      return $country[0];
    }
    #else
    return $country;
  }

  /**
   * Returns string with the country code
   *
   * @param $number
   *   Telephone number in any of following formats(123456789, +123456789, 00123456789)
   *
   * @return
   *   string with country code
   */
  public static function getCodeFromNumber($number) {
    $number = VoipNumber::NormalizeNumber($number);

    $tmp_code = substr($number, 0, 4); //Country code length is max 4 digits
    for($i=strlen($tmp_code); $i>=0; $i--) {
      $code = substr($tmp_code, 0, $i);
      $country_code = db_result(db_query("SELECT code FROM {voipnumber_dialing_codes} WHERE code='%s'", $code));
      if($country_code) {
        return $country_code;
      }
    }
    return NULL;
  }

  /**
   * Returns string with the country name or an array of country names
   *
   * @param $number
   *   Telephone number in any of following formats(123456789, +123456789, 00123456789)
   *
   * @return
   *   string or array with country names
   */
  public static function getCountryFromNumber($number) {
    $code = VoipNumber::getCodeFromNumber($number);
    $country = VoipNumber::getCountryFromCode($code);
    return $country;
  }

  public static function NormalizeNumber($number) {
    //Remove special chars from number
    $chars = array(' ', '-', '.', '(', ')', '+');
    $number = str_replace($chars, '', $number);

    //Remove 00 from begining of number
    $pattern = '/^00/';
    $number = preg_replace($pattern, '', $number);
    return $number;
  }
}

